"""
License: BSD

 van.frag (van.frag@gmail.com)
"""
import datetime
import re
from pkg_resources import resource_filename

# Trac and Genshi imports
from trac.core import *
from trac.config import Option
from trac.config import ListOption, BoolOption, IntOption
from trac.web.chrome import ITemplateProvider
from trac.wiki.formatter import format_to_oneliner

# TracFullBlog imports
from tracfullblog.api import IBlogChangeListener
from tracfullblog.model import BlogPost, BlogComment

from notification import BlogNotifierEmail

class BlogNotifierModule(Component):
    
    implements(ITemplateProvider, IBlogChangeListener)

    # Options
    
    smtp_from = Option(
        'blog-notifier', 'smtp_from', 'trac+wiki@localhost',
        """Sender address to use in notification emails.""")

    from_name = Option(
        'blog-notifier', 'from_name', None,
        """Sender name to use in notification emails.

        Defaults to project name.""")

    smtp_always_bcc = ListOption(
        'blog-notifier', 'smtp_always_bcc', [],
        doc="""Comma separated list of email address(es) to always send
        notifications to.

        Addresses do not appear publicly (Bcc:).""")

    # ITemplateProvider methods

    def get_htdocs_dirs(self):
        """ Makes the 'htdocs' folder inside the egg available. """
        return [('tracfullblog', resource_filename('tracfullblog', 'htdocs'))]

    def get_templates_dirs(self):
        """ Location of Trac templates provided by plugin. """
        return [resource_filename('tracfullblog', 'templates')]

    #IBlogChangeListener

    def blog_post_changed(self, postname, version):
        """Called when a new blog post 'postname' with 'version' is added .
        version==1 denotes a new post, version>1 is a new version on existing post."""
        bp = BlogPost(self.env, postname, version)
        notifier = BlogNotifierEmail(self.env)
        action = 'added'
        if version > 1:
            action = 'changed' 
        notifier.notify(bp, action, version, bp.version_time, bp.version_comment, bp.version_author)

    def blog_post_deleted(self, postname, version, fields):
        """Called when a blog post is deleted:
        version==0 means all versions (or last remaining) version is deleted.
        Any version>0 denotes a specific version only.
        Fields is a dict with the pre-existing values of the blog post.
            If all (or last) the dict will contain the 'current' version contents."""
        bp = BlogPost(self.env, postname, version)
        notifier = BlogNotifierEmail(self.env)
        action = 'deleted'
        author = fields['author']
        if version > 0:
            action = 'deleted_version'
            author = fields['version_author']
        notifier.notify(bp, action, version, datetime.datetime.now(utc), '', author)

    def blog_comment_added(self, postname, number):
        """Called when Blog comment number N on post 'postname' is added."""
        action = 'comment_added'
        bp = BlogPost(self.env, postname, 0)
        bc = BlogComment(self.env, postname, number)
        notifier = BlogNotifierEmail(self.env)
        notifier.notify(bp, action, bp.version, bc.time, bc.comment, bp.author)
        

    def blog_comment_deleted(self, postname, number, fields):
        """Called when blog post comment 'number' is deleted.
        number==0 denotes all comments is deleted and fields will be empty.
            (usually follows a delete of the blog post).
        number>0 denotes a specific comment is deleted, and fields will contain
            the values of the fields as they existed pre-delete."""

